@@ -70,6 +70,7 @@ from data_diff.abcs.database_types import (
     String_Alphanum,
     String_UUID,
     String_VaryingAlphanum,
+    String_VaryingUnicode,
     Struct,
     TemporalType,
     Text,
@@ -316,6 +317,12 @@ class BaseDialect(abc.ABC):
                 return f"CAST('{elem.uuid}' AS UUID)"
             s = f"'{elem.uuid}'"
             return s.upper() if elem.uppercase else s.lower() if elem.lowercase else s
+        elif isinstance(elem, (ArithDateTime, ArithTimestamp, ArithTimestampTZ)):
+            return self.timestamp_value(elem._dt)
+        elif isinstance(elem, ArithDate):
+            from datetime import time
+
+            return self.timestamp_value(datetime.combine(elem._date, time.min))
         elif isinstance(elem, ArithString):
             return f"'{elem}'"
         assert False, elem
@@ -770,7 +777,7 @@ class BaseDialect(abc.ABC):
         elif isinstance(v, ArithTimestampTZ):
             return f"'{str(v)}'"
         elif isinstance(v, ArithDateTime):
-            return f"'{str(v)}'"
+            return self.timestamp_value(v._dt)
         return repr(v)
 
     def constant_values(self, rows) -> str:
@@ -1254,9 +1261,16 @@ class Database(abc.ABC):
                         logger.debug(
                             f"Mixed Alphanum/Non-Alphanum values detected in column {'.'.join(table_path)}.{col_name}. It cannot be used as a key."
                         )
+                        # Fallback to Unicode string type
+                        assert col_name in col_dict
+                        col_dict[col_name] = String_VaryingUnicode(collation=col_dict[col_name].collation)
                     else:
                         assert col_name in col_dict
                         col_dict[col_name] = String_VaryingAlphanum(collation=col_dict[col_name].collation)
+                else:
+                    # All samples failed alphanum test, fallback to Unicode string
+                    assert col_name in col_dict
+                    col_dict[col_name] = String_VaryingUnicode(collation=col_dict[col_name].collation)
 
         return col_dict
 
