@@ -121,13 +121,23 @@ class Dialect(BaseDialect):
     def to_string(self, s: str) -> str:
         s_temp = re.sub(r'["\[\]`]', "", s)
         col_info = self.get_column_raw_info(s_temp)
+
         ch_len = (col_info and col_info.character_maximum_length) or None
-        if not ch_len:
-            ch_len = 2500
-        ch_len = max(ch_len, 2500)
-        if col_info and col_info.data_type in ["nvarchar", "nchar", "ntext"]:
+
+        if ch_len is None or ch_len <= 0:
+            ch_len = "MAX"
+        else:
+            ch_len = min(ch_len, 8000)
+
+        if col_info and col_info.data_type.lower().strip() in ["nvarchar", "nchar"]:
             return f"CONVERT(NVARCHAR({ch_len}), {s})"
 
+        elif col_info and col_info.data_type.lower().strip() == "text":
+            return f"CONVERT(VARCHAR(MAX), {s})"
+
+        elif col_info and col_info.data_type.lower().strip() == "ntext":
+            return f"CONVERT(NVARCHAR(MAX), {s})"
+
         return f"CONVERT(VARCHAR({ch_len}), {s})"
 
     def type_repr(self, t) -> str:
@@ -151,7 +161,6 @@ class Dialect(BaseDialect):
         limit: Optional[int] = None,
         has_order_by: Optional[bool] = None,
     ) -> str:
-        import re
 
         if offset:
             raise NotImplementedError("No support for OFFSET in query")
@@ -208,15 +217,7 @@ class Dialect(BaseDialect):
         return tuple(name.split("."))
 
     def normalize_uuid(self, value, coltype):
-        s_temp = re.sub(r'["\[\]`]', "", value)
-        col_info = self.get_column_raw_info(s_temp)
-        ch_len = (col_info and col_info.character_maximum_length) or None
-        if not ch_len:
-            ch_len = 2500
-        ch_len = max(ch_len, 2500)
-        if isinstance(coltype, String_UUID):
-            return f"CAST({value} AS VARCHAR({ch_len}))"
-        return f"CAST({value} AS VARCHAR(36))"
+        return self.to_string(value)
 
 
 @attrs.define(frozen=False, init=False, kw_only=True)
