@@ -299,7 +299,9 @@ class Dialect(BaseDialect):
             return timestamp_str
 
     def normalize_number(self, value: str, coltype: FractionalType) -> str:
-        return self.to_string(f"CAST({value} AS DECIMAL(38, {coltype.precision}))")
+        # scale = getattr(coltype, "scale", 0) or 0
+        precision = getattr(coltype, "precision", 0) or 0
+        return self.to_string(f"CAST({value} AS DECIMAL(38, {precision}))")
 
     # def md5_as_int(self, s: str) -> str:
     #     """Returns an MD5 hash of the input string as an integer for Sybase IQ."""
@@ -344,7 +346,7 @@ class Dialect(BaseDialect):
 
     def md5_as_int(self, s: str) -> str:
         if self.sybase_driver_type.is_ase or self.query_config_for_free_tds["ase_query_chosen"]:
-            return f"CAST(HEXTOINT(LEFT(CAST(HASH({s}, 'MD5') AS VARCHAR(32)), 8)) AS BIGINT) - 140737488355327"
+            return f"CAST(HEXTOINT(LEFT(CAST(HASH({s}, 'MD5') AS VARCHAR(32)), 8)) AS BIGINT) % 2147483647"
         base_prime = self.get_unique_prime_for_column(s)
         separator = " +\n        "
         parts = [f"LENGTH(COALESCE({s}, '')) * {base_prime}"]
@@ -474,7 +476,7 @@ class Sybase(ThreadedDatabase):
         username = self._args.get("user", None)
         password = self._args.get("password", None)
         driver = self._args.get("driver", None)
-        max_query_timeout = 60 * 5  # 300 seconds
+        max_query_timeout = 60 * 5  # 5 minutes
 
         if self.dialect.sybase_driver_type.is_freetds:
             conn_dict = {
@@ -488,10 +490,10 @@ class Sybase(ThreadedDatabase):
 
             conn_dict["host"] = host or server
             try:
-                logger.debug(f"Attempting FreeTDS connection with config: {conn_dict}")
+                logger.debug("Attempting FreeTDS connection..")
                 self._conn = self._sybase.connect(**conn_dict)
                 self._conn.timeout = max_query_timeout
-                logger.info(f"Successfully connected to Sybase using FreeTDS: {conn_dict}")
+                logger.info("Successfully connected to Sybase using FreeTDS")
                 return self._conn
             except Exception as e:
                 error_msg = f"Failed to connect to Sybase with FreeTDS: {str(e)}"
@@ -565,23 +567,13 @@ class Sybase(ThreadedDatabase):
                     final_config.update(ase_config)
 
                     try:
-                        logger.debug(f"Attempting connection with config: {final_config}")
+                        logger.debug("Attempting connection..")
                         self._conn = self._sybase.connect(**final_config)
                         self._conn.timeout = max_query_timeout
-                        logger.info(
-                            f"Successfully connected to Sybase using: "
-                            f"driver={driver}, "
-                            f"{attempt['key']}={attempt['value']}, "
-                            f"port_config={port_config}, "
-                            f"ase_config={ase_config}"
-                        )
+                        logger.info(f"Successfully connected to Sybase using: driver={driver}")
                         return self._conn
                     except Exception as e:
-                        error_msg = (
-                            f"Failed with {attempt['key']}={attempt['value']}, "
-                            f"port_config={port_config}, "
-                            f"ase_config={ase_config}: {str(e)}"
-                        )
+                        error_msg = "Failed to connect to sybase"
                         logger.debug(error_msg)
                         errors.append(error_msg)
                         continue
