@@ -28,7 +28,7 @@ import attrs
 # logger = getLogger(__name__)
 from loguru import logger
 
-from data_diff.abcs.database_types import IKey
+from data_diff.abcs.database_types import IKey, Integer, StringType
 from data_diff.errors import DataDiffMismatchingKeyTypesError
 from data_diff.info_tree import InfoTree, SegmentInfo
 from data_diff.table_segment import TableSegment, create_mesh_from_points
@@ -340,13 +340,22 @@ class TableDiffer(ThreadBase, ABC):
             if not isinstance(kt, IKey):
                 raise NotImplementedError(f"Cannot use a column of type {kt} as a key")
 
+        mismatched_key_types = False
         for i, (kt1, kt2) in enumerate(safezip(key_types1, key_types2)):
             if kt1.python_type is not kt2.python_type:
-                k1 = table1.key_columns[i]
-                k2 = table2.key_columns[i]
-                raise DataDiffMismatchingKeyTypesError(
-                    f"Key columns {k1} type: {kt1.python_type} and {k2} type: {kt2.python_type} can't be compared due to different types."
-                )
+                # Allow integer vs string, and string vs string variants for diffing, but mark as mismatched
+                if (isinstance(kt1, Integer) and isinstance(kt2, StringType)) or (
+                    isinstance(kt2, Integer) and isinstance(kt1, StringType)
+                ):
+                    mismatched_key_types = True
+                elif isinstance(kt1, StringType) and isinstance(kt2, StringType):
+                    mismatched_key_types = True
+                else:
+                    k1 = table1.key_columns[i]
+                    k2 = table2.key_columns[i]
+                    raise DataDiffMismatchingKeyTypesError(
+                        f"Key columns {k1} type: {kt1.python_type} and {k2} type: {kt2.python_type} can't be compared due to different types."
+                    )
 
         # Query min/max values
         key_ranges = self._threaded_call_as_completed("query_key_range", [table1, table2])
@@ -380,24 +389,27 @@ class TableDiffer(ThreadBase, ABC):
         # Overall, the max number of new regions in this 2nd pass is 3^|k| - 1
 
         # Note: python types can be the same, but the rendering parameters (e.g. casing) can differ.
-        min_key2, max_key2 = self._parse_key_range_result(key_types2, next(key_ranges))
+        # If key types mismatched (e.g., int vs string), skip the second meshing pass to avoid
+        # attempting to sort mixed-type tuples (e.g., ArithAlphanumeric vs int).
+        if not mismatched_key_types:
+            min_key2, max_key2 = self._parse_key_range_result(key_types2, next(key_ranges))
 
-        points = [list(sorted(p)) for p in safezip(min_key1, min_key2, max_key1, max_key2)]
-        box_mesh = create_mesh_from_points(*points)
+            points = [list(sorted(p)) for p in safezip(min_key1, min_key2, max_key1, max_key2)]
+            box_mesh = create_mesh_from_points(*points)
 
-        new_regions = [(p1, p2) for p1, p2 in box_mesh if p1 < p2 and not (p1 >= min_key1 and p2 <= max_key1)]
+            new_regions = [(p1, p2) for p1, p2 in box_mesh if p1 < p2 and not (p1 >= min_key1 and p2 <= max_key1)]
 
-        for p1, p2 in new_regions:
-            extra_table1 = table1.new_key_bounds(min_key=p1, max_key=p2, key_types=key_types1)
-            extra_table2 = table2.new_key_bounds(min_key=p1, max_key=p2, key_types=key_types2)
-            ti.submit(
-                self._bisect_and_diff_segments,
-                ti,
-                extra_table1,
-                extra_table2,
-                info_tree,
-                priority=999,
-            )
+            for p1, p2 in new_regions:
+                extra_table1 = table1.new_key_bounds(min_key=p1, max_key=p2, key_types=key_types1)
+                extra_table2 = table2.new_key_bounds(min_key=p1, max_key=p2, key_types=key_types2)
+                ti.submit(
+                    self._bisect_and_diff_segments,
+                    ti,
+                    extra_table1,
+                    extra_table2,
+                    info_tree,
+                    priority=999,
+                )
 
         return ti
 
