@@ -18,7 +18,7 @@ import time
 from collections import defaultdict
 from numbers import Number
 from threading import Lock
-from typing import Any, Collection, Dict, Iterator, List, Sequence, Set, Tuple
+from typing import Any, Collection, Dict, Iterator, List, Optional, Sequence, Set, Tuple
 
 import attrs
 from loguru import logger
@@ -448,7 +448,39 @@ class HashDiffer(TableDiffer):
     ):
         # Check if level exceeds maximum allowed recursion depth
         if level > 15:
-            raise RecursionError(f"Maximum recursion level exceeded: {level} > 15")
+            logger.warning(
+                ". " * level
+                + f"Maximum recursion level reached ({level}); switching to direct row comparison for segment {table1.min_key}..{table1.max_key}"
+            )
+            # Fallback: download rows and diff locally to prevent excessive recursion
+            rows1, rows2 = self._threaded_call("get_values", [table1, table2])
+            json_cols = {
+                i: colname
+                for i, colname in enumerate(table1.extra_columns)
+                if isinstance(table1._schema[colname], JSON)
+            }
+            diff = list(
+                diff_sets(
+                    rows1,
+                    rows2,
+                    json_cols=json_cols,
+                    columns1=table1.relevant_columns,
+                    columns2=table2.relevant_columns,
+                    key_columns1=table1.key_columns,
+                    key_columns2=table2.key_columns,
+                    ignored_columns1=self.ignored_columns1,
+                    ignored_columns2=self.ignored_columns2,
+                    diff_tracker=self._diff_tracker,
+                )
+            )
+            info_tree.info.set_diff(diff)
+            info_tree.info.rowcounts = {1: len(rows1), 2: len(rows2)}
+            self.stats["rows_downloaded"] = self.stats.get("rows_downloaded", 0) + max(len(rows1), len(rows2))
+            logger.info(
+                ". " * level
+                + f"Diff found {len(diff)} different rows, {self.stats['rows_downloaded']} total rows downloaded."
+            )
+            return diff
 
         # Initialize diff tracker if not already done
         self._initialize_diff_tracker(table1, table2)
@@ -553,7 +585,39 @@ class HashDiffer(TableDiffer):
     ):
         # Check if level exceeds maximum allowed recursion depth
         if level > 15:
-            raise RecursionError(f"Maximum recursion level exceeded: {level} > 15")
+            logger.warning(
+                ". " * level
+                + f"Maximum recursion level reached ({level}); switching to direct row comparison for segment {table1.min_key}..{table1.max_key}"
+            )
+            # Fallback: download rows and diff locally to prevent excessive recursion
+            rows1, rows2 = self._threaded_call("get_values", [table1, table2])
+            json_cols = {
+                i: colname
+                for i, colname in enumerate(table1.extra_columns)
+                if isinstance(table1._schema[colname], JSON)
+            }
+            diff = list(
+                diff_sets(
+                    rows1,
+                    rows2,
+                    json_cols=json_cols,
+                    columns1=table1.relevant_columns,
+                    columns2=table2.relevant_columns,
+                    key_columns1=table1.key_columns,
+                    key_columns2=table2.key_columns,
+                    ignored_columns1=self.ignored_columns1,
+                    ignored_columns2=self.ignored_columns2,
+                    diff_tracker=self._diff_tracker,
+                )
+            )
+            info_tree.info.set_diff(diff)
+            info_tree.info.rowcounts = {1: len(rows1), 2: len(rows2)}
+            self.stats["rows_downloaded"] = self.stats.get("rows_downloaded", 0) + max(len(rows1), len(rows2))
+            logger.info(
+                ". " * level
+                + f"Diff found {len(diff)} different rows, {self.stats['rows_downloaded']} total rows downloaded."
+            )
+            return diff
 
         assert table1.is_bounded and table2.is_bounded
 
@@ -647,3 +711,316 @@ class HashDiffer(TableDiffer):
             return diff
 
         return super()._bisect_and_diff_segments(ti, table1, table2, info_tree, level, max_rows)
+
+
+@attrs.define(frozen=False)
+class HashDiffer(HashDiffer):
+    """
+    Enhanced HashDiffer with in-memory mode support.
+
+    Additional Parameters:
+        in_memory_diff (bool): If True, skip checksums and download segments directly for in-memory comparison.
+                               If False, use standard checksum-based bisection (default behavior).
+        memory_segment_size (int): When in_memory_diff=True, target number of rows per segment before downloading.
+    """
+
+    in_memory_diff: bool = False
+    memory_segment_size: int = 10000
+
+    def __attrs_post_init__(self) -> None:
+        super().__attrs_post_init__()
+
+        if self.in_memory_diff:
+            logger.info("=" * 70)
+            logger.info("IN-MEMORY DIFF MODE ENABLED")
+            logger.info("  - Checksum queries: DISABLED")
+            logger.info(f"  - Segment size: {self.memory_segment_size} rows")
+            logger.info(f"  - Threading: {'ENABLED' if self.threaded else 'DISABLED'}")
+            logger.info(f"  - Egress limit: {self.egress_limit} rows")
+            logger.info("=" * 70)
+
+            # Adjust bisection threshold for in-memory mode
+            if self.memory_segment_size > 0:
+                self.bisection_threshold = self.memory_segment_size
+
+    def _should_skip_checksum_and_download(self, max_rows: int) -> bool:
+        """
+        Determine if we should skip checksum and directly download segment data.
+
+        Returns True if:
+        1. in_memory_diff flag is enabled, OR
+        2. Traditional conditions: segment is below bisection threshold
+        """
+        return self.in_memory_diff
+        # if self.in_memory_diff:
+        #     # In memory mode: download if segment is at or below target size
+        #     return max_rows <= self.memory_segment_size
+        # else:
+        #     # Traditional mode: use bisection threshold
+        #     return self.bisection_disabled or max_rows < self.bisection_threshold
+
+    def _diff_segments(
+        self,
+        ti,
+        table1: TableSegment,
+        table2: TableSegment,
+        info_tree: InfoTree,
+        max_rows: int,
+        level=0,
+        segment_index=None,
+        segment_count=None,
+    ):
+        """
+        Enhanced segment diffing with in-memory mode support.
+        """
+        # Check recursion depth limit
+        if level > 15:
+            logger.warning(
+                ". " * level + f"Maximum recursion level ({level}) reached; "
+                f"downloading segment {table1.min_key}..{table1.max_key}"
+            )
+            return self._download_and_diff_locally(table1, table2, info_tree, level)
+
+        # Initialize diff tracker
+        self._initialize_diff_tracker(table1, table2)
+
+        logger.info(
+            ". " * level + f"Diffing segment {segment_index}/{segment_count}, "
+            f"key-range: {table1.min_key}..{table2.max_key}, "
+            f"size <= {max_rows}"
+        )
+
+        # Check all stop conditions
+        if not self._check_continuation_conditions(table1, info_tree, level):
+            return
+
+        # IN-MEMORY MODE: Skip checksum if flag is set or segment is small enough
+        if self._should_skip_checksum_and_download(max_rows):
+            if self.in_memory_diff:
+                logger.info(". " * level + f"[IN-MEMORY MODE] Downloading segment directly " f"(size: {max_rows} rows)")
+
+            return self._download_and_diff_locally(table1, table2, info_tree, level)
+
+        # STANDARD MODE: Perform checksum-based comparison
+        return self._checksum_and_bisect_if_needed(
+            ti, table1, table2, info_tree, level, max_rows, segment_index, segment_count
+        )
+
+    def _check_continuation_conditions(self, table1: TableSegment, info_tree: InfoTree, level: int) -> bool:
+        """Check if we should continue diffing (respects limits)."""
+
+        # Check per-column diff limit
+        elapsed = time.monotonic() - self.start_time
+        if (
+            len(self._diff_tracker.get_stopped_columns()) > 0
+            and not self.stats.get("rows_downloaded", 0) >= self.egress_limit
+            and not elapsed > self.timeout_limit * 60
+        ):
+            self.update_comparison_tracker(
+                reason_type="per_column_diff_limit",
+                segment=f"{table1.min_key}..{table1.max_key}",
+            )
+
+        if not self._diff_tracker.has_active_targets(len(table1.relevant_columns)):
+            logger.info(
+                "Diffing stopped because columns with potential differences "
+                "have reached their configured diff limits."
+            )
+            info_tree.info.is_diff = False
+            self.update_comparison_tracker(
+                reason_type="per_column_diff_limit",
+                segment=f"{table1.min_key}..{table1.max_key}",
+            )
+            return False
+
+        # Check egress limit
+        if self.stats.get("rows_downloaded", 0) >= self.egress_limit:
+            info_tree.info.is_diff = False
+            logger.info(
+                ". " * level + f"Row download limit reached "
+                f"{self.stats.get('rows_downloaded')}, stopping bisection for "
+                f"segment {table1.min_key}..{table1.max_key}"
+            )
+            self.update_comparison_tracker(
+                reason_type="egress_limit",
+                segment=f"{table1.min_key}..{table1.max_key}",
+            )
+            return False
+
+        # Check timeout
+        elapsed = time.monotonic() - self.start_time
+        if elapsed > self.timeout_limit * 60:
+            info_tree.info.is_diff = False
+            logger.info(
+                ". " * level + f"Timeout limit reached ({self.timeout_limit} min); "
+                f"stopping bisection for segment {table1.min_key}..{table1.max_key}"
+            )
+            self.update_comparison_tracker(
+                reason_type="timeout",
+                segment=f"{table1.min_key}..{table1.max_key}",
+            )
+            return False
+
+        return True
+
+    def _download_and_diff_locally(
+        self,
+        table1: TableSegment,
+        table2: TableSegment,
+        info_tree: InfoTree,
+        level: int,
+    ) -> List:
+        """Download segment rows and perform in-memory diff."""
+        start_time = time.monotonic()
+        mode_label = "[IN-MEMORY]" if self.in_memory_diff else "[STANDARD]"
+        logger.info(
+            ". " * level + f"{mode_label} Downloading rows for comparison: " f"{table1.min_key}..{table1.max_key}"
+        )
+
+        # Download rows from both tables
+        rows1, rows2 = self._threaded_call("get_values", [table1, table2])
+
+        # Update statistics
+        downloaded = max(len(rows1), len(rows2))
+        self.stats["rows_downloaded"] = self.stats.get("rows_downloaded", 0) + downloaded
+
+        logger.info(
+            ". " * level + f"{mode_label} Downloaded {len(rows1)} and {len(rows2)} rows. "
+            f"Total downloaded: {self.stats['rows_downloaded']}"
+            f"Time taken in ms: {int((time.monotonic() - start_time) * 1000)}ms"
+        )
+
+        # Perform in-memory diff
+        json_cols = {
+            i: colname for i, colname in enumerate(table1.extra_columns) if isinstance(table1._schema[colname], JSON)
+        }
+
+        diff = list(
+            diff_sets(
+                rows1,
+                rows2,
+                json_cols=json_cols,
+                columns1=table1.relevant_columns,
+                columns2=table2.relevant_columns,
+                key_columns1=table1.key_columns,
+                key_columns2=table2.key_columns,
+                ignored_columns1=self.ignored_columns1,
+                ignored_columns2=self.ignored_columns2,
+                diff_tracker=self._diff_tracker,
+            )
+        )
+
+        # Update info tree
+        info_tree.info.set_diff(diff)
+        info_tree.info.rowcounts = {1: len(rows1), 2: len(rows2)}
+
+        logger.info(". " * level + f"{mode_label} Found {len(diff)} different rows in this segment")
+
+        return diff
+
+    def _checksum_and_bisect_if_needed(
+        self,
+        ti,
+        table1: TableSegment,
+        table2: TableSegment,
+        info_tree: InfoTree,
+        level: int,
+        max_rows: int,
+        segment_index: Optional[int],
+        segment_count: Optional[int],
+    ):
+        """Perform checksum comparison and bisect if differences found (standard mode)."""
+
+        logger.info(". " * level + "[CHECKSUM MODE] Comparing segment checksums")
+
+        # Get active columns for checksum (excluding stopped columns)
+        active_cols1, active_cols2 = self._get_checksum_columns(table1, table2)
+
+        # Create segments with updated columns for checksum
+        checksum_table1 = self._create_segment_with_updated_columns(table1, active_cols1)
+        checksum_table2 = self._create_segment_with_updated_columns(table2, active_cols2)
+
+        # Perform checksum
+        (count1, checksum1), (count2, checksum2) = self._threaded_call(
+            "count_and_checksum", [checksum_table1, checksum_table2]
+        )
+
+        assert not info_tree.info.rowcounts
+        info_tree.info.rowcounts = {1: count1, 2: count2}
+
+        # Handle empty segments
+        if count1 == 0 and count2 == 0:
+            logger.debug(
+                "Uneven distribution of keys detected in segment {}..{} "
+                "(big gaps in the key column). For better performance, "
+                "we recommend to increase the bisection-threshold.",
+                table1.min_key,
+                table1.max_key,
+            )
+            assert checksum1 is None and checksum2 is None
+            info_tree.info.is_diff = False
+            return
+
+        # Compare checksums
+        if checksum1 == checksum2:
+            logger.info(". " * level + "[CHECKSUM MODE] Checksums match - no differences")
+            info_tree.info.is_diff = False
+            return
+
+        logger.info(". " * level + "[CHECKSUM MODE] Checksums differ - bisecting segment")
+        info_tree.info.is_diff = True
+
+        # Bisect and continue
+        return self._bisect_and_diff_segments(ti, table1, table2, info_tree, level=level, max_rows=max(count1, count2))
+
+    def _bisect_and_diff_segments(
+        self,
+        ti,
+        table1: TableSegment,
+        table2: TableSegment,
+        info_tree: InfoTree,
+        level=0,
+        max_rows=None,
+    ):
+        """
+        Enhanced bisection with in-memory mode support.
+        """
+        # Check recursion limit
+        if level > 15:
+            logger.warning(
+                ". " * level + f"Maximum recursion level ({level}) reached; "
+                f"downloading segment {table1.min_key}..{table1.max_key}"
+            )
+            return self._download_and_diff_locally(table1, table2, info_tree, level)
+
+        assert table1.is_bounded and table2.is_bounded
+
+        # Initialize diff tracker
+        self._initialize_diff_tracker(table1, table2)
+
+        # Check continuation conditions
+        if not self._check_continuation_conditions(table1, info_tree, level):
+            return
+
+        # Calculate max space size
+        max_space_size = max(table1.approximate_size(self.t1_row_count), table2.approximate_size(self.t2_row_count))
+
+        if max_rows is None:
+            max_rows = max_space_size
+
+        info_tree.info.max_rows = max_rows
+
+        # Check if we should download directly
+        should_download = (
+            self.bisection_disabled or max_rows < self.bisection_threshold or max_space_size < self.bisection_factor * 2
+        )
+
+        # In-memory mode: also download if at target segment size
+        if self.in_memory_diff and max_rows <= self.memory_segment_size:
+            should_download = True
+
+        if should_download:
+            return self._download_and_diff_locally(table1, table2, info_tree, level)
+
+        # Otherwise, continue with standard bisection
+        return super()._bisect_and_diff_segments(ti, table1, table2, info_tree, level, max_rows)
