@@ -25,6 +25,7 @@ from typing_extensions import Self
 
 from data_diff.abcs.database_types import DbKey, DbPath, DbTime, IKey, NumericType
 from data_diff.databases.base import Database
+from data_diff.databases.redis import RedisBackend
 from data_diff.queries.api import (
     SKIP,
     Code,
@@ -50,6 +51,7 @@ from data_diff.utils import (
     ArithTimestamp,
     ArithTimestampTZ,
     ArithUnicodeString,
+    JobCancelledError,
     Vector,
     safezip,
     split_space,
@@ -220,6 +222,7 @@ class TableSegment:
             },
         }
     )
+    job_id: Optional[int] = None
 
     def __attrs_post_init__(self) -> None:
         if not self.update_column and (self.min_update or self.max_update):
@@ -252,6 +255,15 @@ class TableSegment:
     def _where(self) -> Optional[str]:
         return f"({self.where})" if self.where else None
 
+    def _column_expr(self, column_name: str) -> Expr:
+        """Return expression for a column, applying configured SQL transform if present."""
+        quoted_column_name = self.database.quote(s=column_name)
+        if self.transform_columns and column_name in self.transform_columns:
+            transform_expr = self.transform_columns[column_name]
+            quoted_expr = transform_expr.format(column=quoted_column_name)
+            return Code(quoted_expr)
+        return this[column_name]
+
     def _with_raw_schema(self, raw_schema: Dict[str, RawColumnInfo]) -> Self:
         schema = self.database._process_table_schema(self.table_path, raw_schema, self.relevant_columns, self._where())
         # return self.new(schema=create_schema(self.database.name, self.table_path, schema, self.case_sensitive))
@@ -277,10 +289,24 @@ class TableSegment:
     def _make_key_range(self):
         if self.min_key is not None:
             for mn, k in safezip(self.min_key, self.key_columns):
-                yield mn <= this[k]
+                quoted = self.database.dialect.quote(k)
+                base_expr_sql = (
+                    self.transform_columns[k].format(column=quoted)
+                    if self.transform_columns and k in self.transform_columns
+                    else quoted
+                )
+                constant_val = self.database.dialect._constant_value(mn)
+                yield Code(f"{base_expr_sql} >= {constant_val}")
         if self.max_key is not None:
             for k, mx in safezip(self.key_columns, self.max_key):
-                yield this[k] < mx
+                quoted = self.database.dialect.quote(k)
+                base_expr_sql = (
+                    self.transform_columns[k].format(column=quoted)
+                    if self.transform_columns and k in self.transform_columns
+                    else quoted
+                )
+                constant_val = self.database.dialect._constant_value(mx)
+                yield Code(f"{base_expr_sql} < {constant_val}")
 
     def _make_update_range(self):
         if self.min_update is not None:
@@ -303,7 +329,8 @@ class TableSegment:
 
         # fetched_cols = [NormalizeAsString(this[c]) for c in self.relevant_columns]
         # select = self.make_select().select(*fetched_cols)
-
+        if self._is_cancelled():
+            raise JobCancelledError(self.job_id)
         select = self.make_select().select(*self._relevant_columns_repr)
         start_time = time.monotonic()
         result = self.database.query(select, List[Tuple])
@@ -343,6 +370,8 @@ class TableSegment:
         Returns:
             list: List of tuples containing the queried row data.
         """
+        if self._is_cancelled():
+            raise JobCancelledError(self.job_id)
         select = self.make_select().select(*self._relevant_columns_repr)
 
         filters = []
@@ -353,9 +382,14 @@ class TableSegment:
                 and_exprs = []
                 for col, val in safezip(self.key_columns, key_values):
                     quoted = self.database.dialect.quote(col)
+                    base_expr_sql = (
+                        self.transform_columns[col].format(column=quoted)
+                        if self.transform_columns and col in self.transform_columns
+                        else quoted
+                    )
                     schema = self._schema[col]
                     if val is None:
-                        and_exprs.append(Code(quoted + " IS NULL"))
+                        and_exprs.append(Code(base_expr_sql + " IS NULL"))
                         continue
                     mk_v = schema.make_value(val)
                     constant_val = self.database.dialect._constant_value(mk_v)
@@ -364,9 +398,9 @@ class TableSegment:
                     if hasattr(self.database.dialect, "timestamp_equality_condition") and hasattr(
                         mk_v, "_dt"
                     ):  # Check if it's a datetime-like object
-                        where_expr = self.database.dialect.timestamp_equality_condition(quoted, constant_val)
+                        where_expr = self.database.dialect.timestamp_equality_condition(base_expr_sql, constant_val)
                     else:
-                        where_expr = f"{quoted} = {constant_val}"
+                        where_expr = f"{base_expr_sql} = {constant_val}"
 
                     and_exprs.append(Code(where_expr))
                 if and_exprs:
@@ -437,18 +471,14 @@ class TableSegment:
         # return [NormalizeAsString(this[c]) for c in self.relevant_columns]
         expressions = []
         for c in self.relevant_columns:
-            quoted_column_name = self.database.quote(s=c)
             schema = self._schema[c]
-            if self.transform_columns and c in self.transform_columns:
-                transform_expr = self.transform_columns[c]
-                quoted_expr = transform_expr.format(column=quoted_column_name)
-                expressions.append(NormalizeAsString(Code(quoted_expr), schema))
-            else:
-                expressions.append(NormalizeAsString(this[c], schema))
+            expressions.append(NormalizeAsString(self._column_expr(c), schema))
         return expressions
 
     def count(self) -> int:
         """Count how many rows are in the segment, in one pass."""
+        if self._is_cancelled():
+            raise JobCancelledError(self.job_id)
         start_time = time.monotonic()
         result = self.database.query(self.make_select().select(Count()), int)
         query_time_ms = (time.monotonic() - start_time) * 1000
@@ -458,12 +488,16 @@ class TableSegment:
 
     def count_and_checksum(self) -> Tuple[int, int]:
         """Count and checksum the rows in the segment, in one pass."""
-
+        if self._is_cancelled():
+            raise JobCancelledError(self.job_id)
         checked_columns = [c for c in self.relevant_columns if c not in self.ignored_columns]
-        # cols = [NormalizeAsString(this[c]) for c in checked_columns]
+        # Build transformed expressions for checksum, honoring transforms and normalization
+        checksum_exprs: List[Expr] = []
+        for c in checked_columns:
+            schema = self._schema[c]
+            checksum_exprs.append(NormalizeAsString(self._column_expr(c), schema))
 
-        # q = self.make_select().select(Count(), Checksum(cols))
-        q = self.make_select().select(Count(), Checksum(self._relevant_columns_repr))
+        q = self.make_select().select(Count(), Checksum(checksum_exprs))
         start_time = time.monotonic()
         count, checksum = self.database.query(q, tuple)
         query_time_ms = (time.monotonic() - start_time) * 1000
@@ -486,7 +520,7 @@ class TableSegment:
         """Query database for minimum and maximum key. This is used for setting the initial bounds."""
         # Normalizes the result (needed for UUIDs) after the min/max computation
         select = self.make_select().select(
-            ApplyFuncAndNormalizeAsString(this[k], f) for k in self.key_columns for f in (min_, max_)
+            ApplyFuncAndNormalizeAsString(self._column_expr(k), f) for k in self.key_columns for f in (min_, max_)
         )
         result = tuple(self.database.query(select, tuple))
 
@@ -527,3 +561,23 @@ class TableSegment:
                 return row_count if row_count is not None else self.count()
         else:
             return row_count if row_count is not None else self.count()
+
+    def _is_cancelled(self) -> bool:
+        run_id = self.job_id
+        if not run_id:
+            return False
+        run_id = f"revoke_job:{run_id}"
+        try:
+            backend = RedisBackend.get_instance()
+            val = backend.client.get(run_id)
+            if not val:
+                return False
+            if isinstance(val, bytes):
+                try:
+                    val = val.decode()
+                except Exception:
+                    val = str(val)
+            return isinstance(val, str) and val.strip().lower() == "revoke"
+        except Exception:
+            logger.warning("Unable to query Redis for cancellation for run_id=%s", run_id)
+            return False
