@@ -171,6 +171,10 @@ def _any_to_uuid(v: Union[str, int, UUID, "ArithUUID"]) -> UUID:
         return v.uuid
     elif isinstance(v, UUID):
         return v
+    # Accept unicode/arithmetic strings that wrap a UUID
+    elif "ArithUnicodeString" in globals() and isinstance(v, ArithUnicodeString):
+        s = getattr(v, "_str", str(v))
+        return UUID(s)
     elif isinstance(v, str):
         return UUID(v)
     elif isinstance(v, int):
@@ -625,6 +629,19 @@ def alphanums_to_numbers(s1: str, s2: str):
     return n1, n2
 
 
+def _alphanum_as_int_for_cmp(s: str) -> Optional[int]:
+    """Interpret an alphanum string as base-10 int if it's purely numeric (optional leading minus).
+
+    Returns None if not purely numeric, in which case callers should fallback to alphanum base ordering.
+    """
+    if re.fullmatch(r"-?\d+", s):
+        try:
+            return int(s)
+        except ValueError:
+            return None
+    return None
+
+
 @attrs.define(frozen=True, eq=False, order=False, repr=False)
 class ArithAlphanumeric(ArithString):
     _str: str
@@ -678,20 +695,53 @@ class ArithAlphanumeric(ArithString):
 
         return NotImplemented
 
-    def __ge__(self, other) -> bool:
-        if not isinstance(other, type(self)):
-            return NotImplemented
-        return self._str >= other._str
-
     def __lt__(self, other) -> bool:
-        if not isinstance(other, type(self)):
-            return NotImplemented
-        return self._str < other._str
+        if isinstance(other, ArithAlphanumeric):
+            return self._str < other._str
+        if isinstance(other, int):
+            v = _alphanum_as_int_for_cmp(self._str)
+            return (v if v is not None else alphanumToNumber(self._str)) < other
+        return NotImplemented
+
+    def __le__(self, other) -> bool:
+        if isinstance(other, ArithAlphanumeric):
+            return self._str <= other._str
+        if isinstance(other, int):
+            v = _alphanum_as_int_for_cmp(self._str)
+            return (v if v is not None else alphanumToNumber(self._str)) <= other
+        return NotImplemented
+
+    def __gt__(self, other) -> bool:
+        if isinstance(other, ArithAlphanumeric):
+            return self._str > other._str
+        if isinstance(other, int):
+            v = _alphanum_as_int_for_cmp(self._str)
+            return (v if v is not None else alphanumToNumber(self._str)) > other
+        return NotImplemented
+
+    def __ge__(self, other) -> bool:
+        if isinstance(other, ArithAlphanumeric):
+            return self._str >= other._str
+        if isinstance(other, int):
+            v = _alphanum_as_int_for_cmp(self._str)
+            return (v if v is not None else alphanumToNumber(self._str)) >= other
+        return NotImplemented
 
     def __eq__(self, other) -> bool:
-        if not isinstance(other, type(self)):
-            return NotImplemented
-        return self._str == other._str
+        if isinstance(other, ArithAlphanumeric):
+            return self._str == other._str
+        if isinstance(other, int):
+            v = _alphanum_as_int_for_cmp(self._str)
+            return (v if v is not None else alphanumToNumber(self._str)) == other
+        return NotImplemented
+
+    def __ne__(self, other) -> bool:
+        if isinstance(other, ArithAlphanumeric):
+            return self._str != other._str
+        if isinstance(other, int):
+            v = _alphanum_as_int_for_cmp(self._str)
+            return (v if v is not None else alphanumToNumber(self._str)) != other
+        return NotImplemented
 
     def new(self, *args, **kw) -> Self:
         return type(self)(*args, **kw, max_len=self._max_len)
@@ -964,3 +1014,9 @@ class SybaseDriverTypes:
     is_ase: bool = False
     is_iq: bool = False
     is_freetds: bool = False
+
+
+class JobCancelledError(RuntimeError):
+    def __init__(self, job_id: str):
+        super().__init__(f"Job ID {job_id} has been revoked.")
+        self.job_id = job_id
