@@ -81,10 +81,12 @@ class DiffAdvancedConfig(BaseModel):
     egress_limit: int = 5_00_000
     per_column_diff_limit: int = 100
     timeout_limit: int = 60 * 5  # minutes
+    in_memory_diff: bool = False  # Whether to perform diff in memory (may use more RAM)
 
 
 class Comparison(BaseModel):
     comparison_name: str
+    job_id: Optional[int] = None
     source: SourceTargetConnection
     target: SourceTargetConnection
     source_columns: Optional[List[str]] = None
@@ -424,9 +426,11 @@ class DataDiffConfig:
                 egress_limit=advanced_diff_config.get("egress_limit", 5_00_000),
                 per_column_diff_limit=advanced_diff_config.get("per_column_diff_limit", 100),
                 timeout_limit=advanced_diff_config.get("timeout_limit", 60 * 5),
+                in_memory_diff=advanced_diff_config.get("in_memory_diff", False),
             )
             new_comparison = {
                 "comparison_name": comparison_name,
+                "job_id": comparison_data.get("job_id", None),
                 "source": self.create_connection_config(
                     source_connection,
                     comparison_data,
