@@ -25,6 +25,7 @@ from rich.console import Console
 
 from data_diff import TableSegment, connect, connect_to_table, diff_tables
 from data_diff.databases import Database
+from data_diff.databases.redis import RedisBackend
 from dcs_sdk.sdk.config.config_loader import Comparison, SourceTargetConnection
 from dcs_sdk.sdk.rules.rules_repository import RulesRepository
 from dcs_sdk.sdk.utils.serializer import serialize_table_schema
@@ -160,6 +161,7 @@ class DBTableDiffer:
             extra_columns=tuple(columns),
             where=where,
             transform_columns=config.transform_columns,
+            job_id=self.config.job_id,
         )
 
     def connect_to_db(self, config: SourceTargetConnection, is_source: bool):
@@ -322,7 +324,6 @@ class DBTableDiffer:
 
             self.table1 = self.connect_to_db_table(self.config.source, is_source=True)
             self.table2 = self.connect_to_db_table(self.config.target, is_source=False)
-
             table_1_sample_data = []
             table_2_sample_data = []
             db1_name = (
@@ -399,6 +400,8 @@ class DBTableDiffer:
                 table_1_sample_data = self.table1.with_schema().get_sample_data(limit=100)
                 sample_keys = [list(row[:pks_len]) for row in table_1_sample_data]
                 table_2_sample_data = self.table2.with_schema().get_sample_data(limit=100, sample_keys=sample_keys)
+                # if self.config.advanced_configuration.in_memory_diff:
+                #     self.config.advanced_configuration.egress_limit = min(max_row_count, 50_00_000)
                 self.diff_iter = diff_tables(
                     self.table1,
                     self.table2,
@@ -410,6 +413,7 @@ class DBTableDiffer:
                     per_column_diff_limit=self.config.advanced_configuration.per_column_diff_limit,
                     egress_limit=self.config.advanced_configuration.egress_limit,
                     timeout_limit=self.config.advanced_configuration.timeout_limit,
+                    in_memory_diff=self.config.advanced_configuration.in_memory_diff,
                 )
 
             columns_mappings = [
@@ -655,7 +659,15 @@ class DBTableDiffer:
                                 break
 
                     def sort_key(row, key_columns, extra_columns):
-                        return tuple(row[k] for k in key_columns + extra_columns if k in row)
+                        key_values = []
+                        for k in key_columns + extra_columns:
+                            if k in row:
+                                value = row[k]
+                                if value is None:
+                                    key_values.append("None")
+                                else:
+                                    key_values.append(value)
+                        return tuple(key_values)
 
                     remaining_sources = [row for i, row in enumerate(source_rows) if i not in used_sources]
                     remaining_targets = [row for j, row in enumerate(target_rows) if j not in used_targets]
@@ -742,6 +754,8 @@ class DBTableDiffer:
 
         safe_close(self.source_db)
         safe_close(self.target_db)
+        if self.config.job_id:
+            safe_close(RedisBackend.get_instance())
 
     def cleanup_duckdb(self, src: str, target: str):
         if src and src.endswith("duckdb"):
