@@ -240,32 +240,30 @@ def differ_rows(
             table_data.append(obj)
         pk_value = tuple(column_values[col] for col in pk_key_cols)
 
-        if sign == "-" and pk_value in source_duplicates:
-            total_source_duplicates += 1
-            if limit is None or len(duplicates_in_source) < limit:
-                masked_obj = apply_masking(obj, src_masking_cols, masking_character)
-                duplicates_in_source.append(masked_obj)
-            continue
-
-        if sign == "+" and pk_value in target_duplicates:
-            total_target_duplicates += 1
-            if limit is None or len(duplicates_in_target) < limit:
-                masked_obj = apply_masking(obj, tgt_masking_cols, masking_character)
-                duplicates_in_target.append(masked_obj)
-            continue
-
         if sign == "-" and pk_value in exclusive_source_set:
             if pk_value not in seen_ex_source and (limit is None or len(exclusive_to_source) < limit):
                 masked_obj = apply_masking(obj, src_masking_cols, masking_character)
                 exclusive_to_source.append(masked_obj)
                 seen_ex_source.add(pk_value)
 
-        elif sign == "+" and pk_value in exclusive_target_set:
+        if sign == "+" and pk_value in exclusive_target_set:
             if pk_value not in seen_ex_target and (limit is None or len(exclusive_to_target) < limit):
                 masked_obj = apply_masking(obj, tgt_masking_cols, masking_character)
                 exclusive_to_target.append(masked_obj)
                 seen_ex_target.add(pk_value)
 
+        if sign == "-" and pk_value in source_duplicates:
+            total_source_duplicates += 1
+            if limit is None or len(duplicates_in_source) < limit:
+                masked_obj = apply_masking(obj, src_masking_cols, masking_character)
+                duplicates_in_source.append(masked_obj)
+
+        if sign == "+" and pk_value in target_duplicates:
+            total_target_duplicates += 1
+            if limit is None or len(duplicates_in_target) < limit:
+                masked_obj = apply_masking(obj, tgt_masking_cols, masking_character)
+                duplicates_in_target.append(masked_obj)
+
         if pk_value in diff_pks_to_collect:
             if pk_value not in diff_records_dict:
                 diff_records_dict[pk_value] = []
